// lcd.c funkcje obsugi wyswietlacza

#include <avr\io.h>
#include <avr\pgmspace.h>
#include <inttypes.h>
#include <stdlib.h>
#include "harddef.h"
#include "makra.h"
#include "delay.h"
#include "lcd.h"

//funkcje niskiego poziomu, wyslanie bajtu
#define LCD_EPULSE() \
	{PORT(LCD_PORT) |= 1<<LCD_E; \
	delay250ns(); \
	PORT(LCD_PORT) &= ~(1<<LCD_E);}
	
void lcd_sendHalf(uint8_t data)		//modsza cz
{
	data = (data & 0x0F) << LCD_D4;
	PORT(LCD_PORT) = (PORT(LCD_PORT) & ~(0x0F<<LCD_D4)) | data;
	LCD_EPULSE();
}

void lcd_send(uint8_t data)
{
	//starsza cz
	lcd_sendHalf(data>>4);
	//modsza cz
	lcd_sendHalf(data);
	delayus8(120);
}

//funkcje interfejsu
void lcd_command(uint8_t command)
{
	PORT(LCD_PORT) &= ~(1<<LCD_RS);
	lcd_send(command);
}

void lcd_data(uint8_t data)
{
	PORT(LCD_PORT) |= 1<<LCD_RS;
	lcd_send(data);
}

void lcd_cls(void)
{
	lcd_command(LCDC_CLS);
	delay100us8(48);
}

void lcd_home(void)
{
	lcd_command(LCDC_HOME);
	delay100us8(48);
}

void lcd_def_sym(void)
{
	lcd_command(0x40);
	// 
	lcd_data(2);	
	lcd_data(4);
	lcd_data(15);
	lcd_data(16);
	lcd_data(14);	
	lcd_data(1);
	lcd_data(30);
	lcd_data(0);
 	// 
	lcd_data(2);	
	lcd_data(4);
	lcd_data(14);
	lcd_data(16);
	lcd_data(16);	
	lcd_data(17);
	lcd_data(14);
	lcd_data(0);
}

void lcd_sp(uint8_t zm)
{
	lcd_data(zm);
}

//inicjacja lcd
void lcd_init(void)
{
	delay100us8(150);
	PORT(LCD_PORT) &= ~(1<<LCD_RS);
	lcd_sendHalf(LCDC_FUNC|LCDC_FUNC8b); 
	delay100us8(41);
	lcd_sendHalf(LCDC_FUNC|LCDC_FUNC8b);
	delay100us8(2);
	lcd_sendHalf(LCDC_FUNC|LCDC_FUNC4b);
	delay100us8(2);

	lcd_command(LCDC_FUNC|LCDC_FUNC4b|LCDC_FUNC2L|LCDC_FUNC5x7);
	lcd_command(LCDC_ON);
	lcd_cls();
	lcd_command(LCDC_MODE|LCDC_MODER);
	lcd_command(LCDC_ON|LCDC_ONDISPLAY);
}

// Funkcje piszce
void lcd_str_P(prog_char* str)
{
	char znak;
	while( 0 != (znak = pgm_read_byte(str++)) )
	lcd_data(znak);
}

void lcd_str(char* str)
{
	char znak;
	while( 0 != (znak = *(str++)) )
	lcd_data(znak);
}

// Wypisuje liczb na wywietlaczu
void lcd_dec(int val)
{
	char bufor[7];
	lcd_str(itoa(val, bufor, 10));
}
